import openmdao.api as om

from aviary.subsystems.geometry.flops_based.utils import (
    calc_lifting_surface_scaler, thickness_to_chord_scaler)
from aviary.utils.aviary_values import AviaryValues
from aviary.variable_info.functions import add_aviary_input, add_aviary_output
from aviary.variable_info.variables import Aircraft


class Canard(om.ExplicitComponent):
    def initialize(self):
        self.options.declare(
            'aviary_options', types=AviaryValues,
            desc='collection of Aircraft/Mission specific options')

    def setup(self):
        add_aviary_input(self, Aircraft.Canard.AREA, 0.0)
        add_aviary_input(self, Aircraft.Canard.THICKNESS_TO_CHORD, 0.0)
        add_aviary_input(self, Aircraft.Canard.WETTED_AREA_SCALER, 1.0)

        add_aviary_output(self, Aircraft.Canard.WETTED_AREA, 0.0)

    def setup_partials(self):
        self.declare_partials(
            Aircraft.Canard.WETTED_AREA,
            [
                Aircraft.Canard.AREA, Aircraft.Canard.THICKNESS_TO_CHORD,
                Aircraft.Canard.WETTED_AREA_SCALER,
            ]
        )

    def compute(
        self, inputs, outputs, discrete_inputs=None, discrete_outputs=None
    ):
        area = inputs[Aircraft.Canard.AREA]

        if area <= 0.0:
            outputs[Aircraft.Canard.WETTED_AREA] = 0.0

            return

        thickness_to_chord = inputs[Aircraft.Canard.THICKNESS_TO_CHORD]
        XMULTC = calc_lifting_surface_scaler(thickness_to_chord)
        scaler = inputs[Aircraft.Canard.WETTED_AREA_SCALER]

        wetted_area = scaler * XMULTC * area

        outputs[Aircraft.Canard.WETTED_AREA] = wetted_area

    def compute_partials(self, inputs, J, discrete_inputs=None):
        area = inputs[Aircraft.Canard.AREA]

        thickness_to_chord = inputs[Aircraft.Canard.THICKNESS_TO_CHORD]
        XMULTC = calc_lifting_surface_scaler(thickness_to_chord)
        scaler = inputs[Aircraft.Canard.WETTED_AREA_SCALER]

        J[
            Aircraft.Canard.WETTED_AREA,
            Aircraft.Canard.AREA
        ] = scaler * XMULTC

        J[
            Aircraft.Canard.WETTED_AREA,
            Aircraft.Canard.THICKNESS_TO_CHORD
        ] = scaler * thickness_to_chord_scaler * area

        J[
            Aircraft.Canard.WETTED_AREA,
            Aircraft.Canard.WETTED_AREA_SCALER
        ] = XMULTC * area




from aviary.utils.aviary_values import AviaryValues
from aviary.variable_info.variables import Aircraft


def check_fold_location_definition(inputs, options: AviaryValues):
    choose_fold_location = options.get_val(
        Aircraft.Wing.CHOOSE_FOLD_LOCATION, units='unitless')
    has_strut = options.get_val(Aircraft.Wing.HAS_STRUT, units='unitless')
    if not choose_fold_location and not has_strut:
        raise RuntimeError(
            "The option CHOOSE_FOLD_LOCATION can only be False if the option HAS_STRUT is True.")

# Possible TODO
# Aircraft.Design.ULF_CALCULATED_FROM_MANEUVER - Aircraft.Design.PART25_STRUCTURAL_CATEGORY
# Aircraft.Engine.FUSELAGE_MOUNTED - Aircraft.Engine.WING_LOCATIONS
# Aircraft.Engine.NUM_ENGINES - Aircraft.Engine.NUM_FUSELAGE_ENGINES - Aircraft.Engine.NUM_WING_ENGINES
# Aircraft.Propulsion.TOTAL_NUM_ENGINES - Aircraft.Propulsion.TOTAL_NUM_FUSELAGE_ENGINES - Aircraft.Propulsion.TOTAL_NUM_WING_ENGINES
# Aircraft.Engine.TYPE - Aircraft.Engine.HAS_PROPELLERS
# Aircraft.Design.COMPUTE_TAIL_VOLUME_COEFFS
# Aircraft.Engine.REFERENCE_WEIGHT
# Aircraft.Fuselage.PROVIDE_SURFACE_AREA - Aircraft.Fuselage.WETTED_AREA_FACTOR
# Mission.Taxi.MACH - pycycle
# Aircraft.HorizontalTail.AREA
# Aircraft.VerticalTail.AREA
# Aircraft.Strut.DIMENSIONAL_LOCATION_SPECIFIED
# Aircraft.Wing.FOLD_LOCATION_IS_DIMENSIONAL
# Aircraft.Wing.HAS_FOLD



import os

IGNORE_LIST = []

packages = [
    'interface',
    'utils',
    'variable_info',
]

index_top = """
# Source Docs

"""


def header(filename, path):

    header = """---
orphan: true
---

# %s

```{eval-rst}
    .. automodule:: %s
        :members:
        :undoc-members:
        :special-members: __init__, __contains__, __iter__, __setitem__, __getitem__
        :show-inheritance:
        :noindex:
```
""" % (filename, path)
    return header


def build_src_docs(top, src_dir, project_name='aviary'):
    doc_dir = os.path.join(top, "_srcdocs")

    if not os.path.isdir(doc_dir):
        os.mkdir(doc_dir)

    packages_dir = os.path.join(doc_dir, "packages")
    if not os.path.isdir(packages_dir):
        os.mkdir(packages_dir)

    index_filename = os.path.join(doc_dir, "index.md")
    index = open(index_filename, "w")
    index_data = index_top

    for package in packages:
        sub_packages = []
        package_filename = os.path.join(packages_dir, package + ".md")
        package_name = project_name + "." + package

        package_dir = os.path.join(src_dir, package.replace('.', '/'))
        for sub_listing in sorted(os.listdir(package_dir)):
            if (os.path.isdir(sub_listing) and sub_listing != "tests") or \
                    (sub_listing.endswith(".py") and not sub_listing.startswith('_')):
                sub_packages.append(sub_listing.rsplit('.')[0])

        if len(sub_packages) > 0:
            title = f"[{package}]"
            link = f"(packages/{package}.md)\n"
            index_data += f"- {title}{link}"

            package_dir = os.path.join(packages_dir, package)
            os.mkdir(package_dir)

            package_file = open(package_filename, "w")
            package_data = f"---\norphan: true\n---\n\n# {package_name}\n\n"

            for sub_package in sub_packages:
                SKIP_SUBPACKAGES = ['__pycache__']
                if sub_package not in SKIP_SUBPACKAGES:
                    title = f"[{sub_package}]"
                    link = f"({package}/{sub_package}.md)\n"
                    package_data += f"- {title}{link}"

                    ref_sheet_filename = os.path.join(
                        package_dir, sub_package + ".md")
                    ref_sheet = open(ref_sheet_filename, "w")

                    filename = sub_package + ".py"
                    ref_sheet.write(header(filename, package_name + "." + sub_package))
                    ref_sheet.close()

            package_file.write(package_data)
            package_file.close()

    index.write(index_data)
    index.close()


if __name__ == '__main__':
    build_src_docs(".", "..")



import openmdao.api as om

from aviary.mission.flops_based.phases.simplified_landing import LandingGroup
from aviary.variable_info.variables import Aircraft, Mission


class Landing:
    """
    Define user constraints for a climb phase.
    Parameters
    ----------
    ref_wing_area : float (0.0)
        reference are of wing (ft^2)
    Cl_max_ldg : float (None)
        maximum coefficient of lift in landing configuration (None)

    Returns
    -------
    Group
        a Group object in OpenMDAO
    """

    def __init__(
        self,
        ref_wing_area=None,
        Cl_max_ldg=None,
    ):
        self.ref_wing_area = ref_wing_area  # ft**2
        self.Cl_max_ldg = Cl_max_ldg  # no units
        # note: need to clean this up so that some of these variables come from
        # connections.

    __slots__ = (
        "ref_wing_area",
        "Cl_max_ldg",
    )

    def build_phase(self, use_detailed=False):
        """
        Construct and return a new phase for landing analysis.
        Parameters
        ----------
        use_detailed : bool (False)
            tells whether to use simplified or detailed landing. Currently detailed is
            disabled.
        Returns
        -------
        Group
            a group in OpenMDAO
        """

        if use_detailed:
            raise om.AnalysisError(
                "Must set landing method to `use_detailed=False`, detailed landing is"
                " not currently enabled."
            )

        ##############
        # Add Inputs #
        ##############

        landing = LandingGroup()
        landing.set_input_defaults(
            Aircraft.Wing.AREA, val=self.ref_wing_area, units="ft**2"
        )
        landing.set_input_defaults(
            Mission.Landing.LIFT_COEFFICIENT_MAX, val=self.Cl_max_ldg,
            units='unitless')

        return landing


if __name__ == "__main__":
    landing_options = Landing(
        ref_wing_area=1370.0,  # ft**2
        Cl_max_ldg=3,  # no units
    )

    landing = landing_options.build_phase(False)

    prob = om.Problem()
    prob.model = landing
    prob.setup(force_alloc_complex=True)
    prob.run_model()
    prob.check_partials(method="cs", compact_print=True)



"""
Define subsystem builder for Aviary core aerodynamics.

Classes
-------
AerodynamicsBuilderBase : the interface for an aerodynamics subsystem builder.

CoreAerodynamicsBuilder : the interface for Aviary's core aerodynamics subsystem builder
"""
import openmdao.api as om

from aviary.variable_info.variables import Aircraft, Mission, Dynamic

from aviary.subsystems.subsystem_builder_base import SubsystemBuilderBase
from aviary.subsystems.aerodynamics.flops_based.aero_report import AeroReport
from aviary.subsystems.aerodynamics.flops_based.design import Design
from aviary.subsystems.aerodynamics.gasp_based.premission_aero import PreMissionAero
from aviary.subsystems.aerodynamics.gasp_based.gaspaero import CruiseAero
from aviary.subsystems.aerodynamics.gasp_based.gaspaero import LowSpeedAero
from aviary.subsystems.aerodynamics.gasp_based.table_based import CruiseAero as TabularCruiseAero
from aviary.subsystems.aerodynamics.gasp_based.table_based import LowSpeedAero as TabularLowSpeedAero
from aviary.subsystems.aerodynamics.flops_based.computed_aero_group import \
    ComputedAeroGroup
from aviary.subsystems.aerodynamics.flops_based.takeoff_aero_group import \
    TakeoffAeroGroup
from aviary.subsystems.aerodynamics.flops_based.solved_alpha_group import \
    SolvedAlphaGroup
from aviary.subsystems.aerodynamics.flops_based.tabular_aero_group import \
    TabularAeroGroup
from aviary.variable_info.enums import LegacyCode


GASP = LegacyCode.GASP
FLOPS = LegacyCode.FLOPS

_default_name = 'aerodynamics'


class AerodynamicsBuilderBase(SubsystemBuilderBase):
    def __init__(self, name=None, meta_data=None):
        if name is None:
            name = _default_name

        super().__init__(name=name, meta_data=meta_data)

    def mission_inputs(self, **kwargs):
        return ['*']

    def mission_outputs(self, **kwargs):
        return ['*']


class CoreAerodynamicsBuilder(AerodynamicsBuilderBase):
    def __init__(self, name=None, meta_data=None, code_origin=None):
        if name is None:
            name = 'core_aerodynamics'

        if code_origin not in (FLOPS, GASP):
            raise ValueError('Code origin is not one of the following: (FLOPS, GASP)')

        self.code_origin = code_origin

        super().__init__(name=name, meta_data=meta_data)

    def build_pre_mission(self, aviary_inputs):
        code_origin = self.code_origin

        if code_origin is GASP:
            aero_group = PreMissionAero(aviary_options=aviary_inputs)

        elif code_origin is FLOPS:
            aero_group = om.Group()
            aero_group.add_subsystem(
                'design', Design(aviary_options=aviary_inputs),
                promotes_inputs=['*'],
                promotes_outputs=['*'])

            aero_group.add_subsystem(
                'aero_report', AeroReport(aviary_options=aviary_inputs),
                promotes_inputs=['*'],
                promotes_outputs=['*'])

        return aero_group

    def build_mission(self, num_nodes, aviary_inputs, **kwargs):
        try:
            method = kwargs.pop('method')
        except KeyError:
            method = None
        if self.code_origin is FLOPS:
            if method is None:
                aero_group = ComputedAeroGroup(num_nodes=num_nodes,
                                               aviary_options=aviary_inputs)

            elif method == 'computed':
                aero_group = ComputedAeroGroup(num_nodes=num_nodes,
                                               aviary_options=aviary_inputs,
                                               **kwargs)

            elif method == 'low_speed':
                aero_group = TakeoffAeroGroup(num_nodes=num_nodes,
                                              aviary_options=aviary_inputs,
                                              **kwargs)

            # TODO solved alpha belongs in the GASP side, rolled into tabular aero
            #      It is currently only here because it is not possible to define
            #      per-subsystem code origins in AviaryProblem yet
            elif method == 'solved_alpha':
                aero_group = SolvedAlphaGroup(num_nodes=num_nodes,
                                              aero_data=kwargs.pop('aero_data'),
                                              **kwargs)

            elif method == 'tabular':
                aero_group = TabularAeroGroup(num_nodes=num_nodes,
                                              CD0_data=kwargs.pop('CD0_data'),
                                              CDI_data=kwargs.pop('CDI_data'),
                                              **kwargs)

            else:
                raise ValueError('FLOPS-based aero method is not one of the following: '
                                 '(computed, low_speed, solved_alpha, tabular)')

        elif self.code_origin is GASP:
            if method is None:
                aero_group = CruiseAero(num_nodes=num_nodes)

            elif method == 'cruise':
                if 'aero_data' in kwargs:
                    aero_group = TabularCruiseAero(num_nodes=num_nodes,
                                                   aero_data=kwargs.pop('aero_data'),
                                                   **kwargs)
                else:
                    aero_group = CruiseAero(num_nodes=num_nodes,
                                            **kwargs)

            elif method == 'low_speed':
                if any(key in kwargs for key in ['free_aero_data',
                                                 'free_flaps_data',
                                                 'free_ground_data']) in kwargs:
                    aero_group = TabularLowSpeedAero(num_nodes=num_nodes,
                                                     free_aero_data=kwargs.pop(
                                                         'free_aero_data'),
                                                     free_flaps_data=kwargs.pop(
                                                         'free_flaps_data'),
                                                     free_ground_data=kwargs.pop(
                                                         'free_ground_data'),
                                                     **kwargs)

                else:
                    aero_group = LowSpeedAero(num_nodes=num_nodes,
                                              **kwargs)

            else:
                raise ValueError('GASP-based aero method is not one of the following: '
                                 '(cruise, low_speed)')

        return aero_group

    def mission_inputs(self, **kwargs):
        method = kwargs['method']
        promotes = ['*']

        if self.code_origin is FLOPS:
            if method == 'computed':
                promotes = [Dynamic.Mission.STATIC_PRESSURE,
                            Dynamic.Mission.MACH,
                            Dynamic.Mission.TEMPERATURE,
                            Dynamic.Mission.MASS,
                            'aircraft:*', 'mission:*']

            elif method == 'solved_alpha':
                promotes = [Dynamic.Mission.ALTITUDE,
                            Dynamic.Mission.MACH,
                            Dynamic.Mission.MASS,
                            Dynamic.Mission.STATIC_PRESSURE,
                            'aircraft:*']

            elif method == 'low_speed':
                promotes = ['angle_of_attack',
                            Dynamic.Mission.ALTITUDE,
                            Dynamic.Mission.FLIGHT_PATH_ANGLE,
                            Mission.Takeoff.DRAG_COEFFICIENT_MIN,
                            Aircraft.Wing.ASPECT_RATIO,
                            Aircraft.Wing.HEIGHT,
                            Aircraft.Wing.SPAN,
                            Dynamic.Mission.DYNAMIC_PRESSURE,
                            Aircraft.Wing.AREA]

            elif method == 'tabular':
                promotes = [Dynamic.Mission.ALTITUDE,
                            Dynamic.Mission.MACH,
                            Dynamic.Mission.MASS,
                            Dynamic.Mission.VELOCITY,
                            Dynamic.Mission.DENSITY,
                            'aircraft:*']

            else:
                raise ValueError('FLOPS-based aero method is not one of the following: '
                                 '(computed, low_speed, solved_alpha, tabular)')

        elif self.code_origin is GASP:
            if method == 'low_speed':
                promotes = ['*',
                            ("airport_alt", Mission.Takeoff.AIRPORT_ALTITUDE),
                            ("CL_max_flaps", Mission.Takeoff.LIFT_COEFFICIENT_MAX),
                            ("dCL_flaps_model", Mission.Takeoff.LIFT_COEFFICIENT_FLAP_INCREMENT),
                            ("dCD_flaps_model", Mission.Takeoff.DRAG_COEFFICIENT_FLAP_INCREMENT),
                            ("flap_defl", Aircraft.Wing.FLAP_DEFLECTION_TAKEOFF)]

            elif method == 'cruise':
                if 'output_alpha' in kwargs:
                    if kwargs['output_alpha']:
                        promotes = ['*', ("lift_req", "weight")]

            else:
                raise ValueError('GASP-based aero method is not one of the following: '
                                 '(low_speed, cruise)')

        return promotes

    def mission_outputs(self, **kwargs):
        method = kwargs['method']
        promotes = ['*']

        if self.code_origin is FLOPS:
            promotes = [Dynamic.Mission.DRAG]
            if method == 'low_speed':
                promotes.append(Dynamic.Mission.LIFT)

        elif self.code_origin is GASP:
            if method == 'low_speed':
                promotes = [Dynamic.Mission.DRAG,
                            Dynamic.Mission.LIFT,
                            'CL', 'CD', 'flap_factor', 'gear_factor']

            elif method == 'cruise':
                if 'output_alpha' in kwargs:
                    if kwargs['output_alpha']:
                        promotes = [Dynamic.Mission.DRAG,
                                    Dynamic.Mission.LIFT,
                                    'alpha']
                else:
                    promotes = [Dynamic.Mission.DRAG,
                                Dynamic.Mission.LIFT,
                                'CL_max']

            else:
                raise ValueError('GASP-based aero method is not one of the following: '
                                 '(low_speed, cruise)')

        return promotes

    def report(self, prob, reports_folder, **kwargs):
        """
        Generate the report for Aviary core aerodynamics analysis

        Parameters
        ----------
        prob : AviaryProblem
            The AviaryProblem that will be used to generate the report
        reports_folder : Path
            Location of the subsystems_report folder this report will be placed in
        """
        if self.code_origin is FLOPS:
            # FLOPS aero report goes here
            return
        elif self.code_origin is GASP:
            # GASP aero report goes here
            return



import unittest

import numpy as np
import openmdao.api as om
from openmdao.utils.assert_utils import (assert_check_partials,
                                         assert_near_equal)
from parameterized import parameterized

from aviary.subsystems.aerodynamics.flops_based.computed_aero_group import ComputedDrag
from aviary.subsystems.aerodynamics.flops_based.drag import SimpleDrag, TotalDrag
from aviary.utils.aviary_values import AviaryValues
from aviary.validation_cases.validation_tests import (get_flops_case_names,
                                                      get_flops_inputs,
                                                      print_case)
from aviary.variable_info.variables import Aircraft, Dynamic

data_sets = get_flops_case_names(
    only=['LargeSingleAisle1FLOPS', 'LargeSingleAisle2FLOPS', 'N3CC'])


class SimpleDragTest(unittest.TestCase):

    @parameterized.expand(data_sets, name_func=print_case)
    def test_case(self, case_name):
        flops_inputs = get_flops_inputs(case_name)

        dynamics_data: AviaryValues = dynamics_test_data[case_name]

        # area = 4 digits precision
        # FCDSUB - 2 digits precision
        # FCDSUP - 2 digits precision
        inputs_keys = (
            Aircraft.Wing.AREA, Aircraft.Design.SUBSONIC_DRAG_COEFF_FACTOR,
            Aircraft.Design.SUPERSONIC_DRAG_COEFF_FACTOR)

        # dynamic pressure = 4 digits precision
        # drag coefficient = 5 digits precision
        dynamics_keys = (Dynamic.Mission.DYNAMIC_PRESSURE,
                         'drag_coefficient', Dynamic.Mission.MACH)

        # drag = 4 digits precision
        outputs_keys = (Dynamic.Mission.DRAG,)

        # using lowest precision from all available data should "always" work
        # - will a higher precision comparison work? find a practical tolerance that fits
        #   the data set
        tol = 2e-4

        prob = om.Problem()
        model = prob.model

        q, _ = dynamics_data.get_item(Dynamic.Mission.DYNAMIC_PRESSURE)
        nn = len(q)
        model.add_subsystem('simple_drag', SimpleDrag(num_nodes=nn), promotes=['*'])

        prob.setup(force_alloc_complex=True)

        for key in inputs_keys:
            val, units = flops_inputs.get_item(key)
            prob.set_val(key, val, units)

        for key in dynamics_keys:
            val, units = dynamics_data.get_item(key)
            prob.set_val(key, val, units)

        prob.run_model()

        for key in outputs_keys:
            try:
                desired, units = dynamics_data.get_item(key)
                actual = prob.get_val(key, units)

                assert_near_equal(actual, desired, tol)

            except ValueError as error:
                msg = f'"{key}": {error!s}'

                raise ValueError(msg) from None

        data = prob.check_partials(out_stream=None, method="cs")
        assert_check_partials(data, atol=2.5e-10, rtol=1e-12)


class TotalDragTest(unittest.TestCase):

    @parameterized.expand(data_sets, name_func=print_case)
    def test_case(self, case_name):
        flops_inputs = get_flops_inputs(case_name)

        dynamics_data: AviaryValues = dynamics_test_data[case_name]

        # area = 4 digits precision
        # FCDSUB - 2 digits precision
        # FCDSUP - 2 digits precision
        inputs_keys = (
            Aircraft.Wing.AREA, Aircraft.Design.SUBSONIC_DRAG_COEFF_FACTOR,
            Aircraft.Design.SUPERSONIC_DRAG_COEFF_FACTOR,
            Aircraft.Design.ZERO_LIFT_DRAG_COEFF_FACTOR,
            Aircraft.Design.LIFT_DEPENDENT_DRAG_COEFF_FACTOR)

        # dynamic pressure = 4 digits precision
        # drag coefficient = 5 digits precision
        dynamics_keys = (Dynamic.Mission.DYNAMIC_PRESSURE,
                         Dynamic.Mission.MACH, 'CD0', 'CDI')

        # drag = 4 digits precision
        outputs_keys = ('drag_coefficient', Dynamic.Mission.DRAG)

        # using lowest precision from all available data should "always" work
        # - will a higher precision comparison work? find a practical tolerance that fits
        #   the data set
        tol = 2e-4

        prob = om.Problem()
        model = prob.model

        q, _ = dynamics_data.get_item(Dynamic.Mission.DYNAMIC_PRESSURE)
        nn = len(q)
        model.add_subsystem('total_drag', TotalDrag(num_nodes=nn), promotes=['*'])

        prob.setup(force_alloc_complex=True)

        for key in inputs_keys:
            val, units = flops_inputs.get_item(key)
            prob.set_val(key, val, units)

        for key in dynamics_keys:
            val, units = dynamics_data.get_item(key)
            prob.set_val(key, val, units)

        prob.run_model()

        for key in outputs_keys:
            try:
                desired, units = dynamics_data.get_item(key)
                actual = prob.get_val(key, units)

                assert_near_equal(actual, desired, tol)

            except ValueError as error:
                msg = f'"{key}": {error!s}'

                raise ValueError(msg) from None

        data = prob.check_partials(out_stream=None, method="cs")
        assert_check_partials(data, atol=2.5e-10, rtol=1e-12)


class ComputedDragTest(unittest.TestCase):

    def test_derivs(self):
        nn = 2

        cdp = np.array([2.5, 1.3])
        cdc = np.array([0.3, 0.7])
        cdi = np.array([1.5, 4.3])
        cdf = np.array([0.534, 0.763])
        M = np.array([0.8, 1.2])

        prob = om.Problem()
        model = prob.model

        model.add_subsystem(
            'computed_drag', ComputedDrag(num_nodes=nn), promotes_inputs=['*'])

        prob.setup(force_alloc_complex=True)

        prob.set_val('skin_friction_drag_coeff', cdf)
        prob.set_val('pressure_drag_coeff', cdp)
        prob.set_val('compress_drag_coeff', cdc)
        prob.set_val('induced_drag_coeff', cdi)
        prob.set_val(Dynamic.Mission.MACH, M)

        prob.set_val(Aircraft.Design.ZERO_LIFT_DRAG_COEFF_FACTOR, 0.7)
        prob.set_val(Aircraft.Design.LIFT_DEPENDENT_DRAG_COEFF_FACTOR, 0.3)
        prob.set_val(Aircraft.Design.SUBSONIC_DRAG_COEFF_FACTOR, 1.4)
        prob.set_val(Aircraft.Design.SUPERSONIC_DRAG_COEFF_FACTOR, 1.1)
        prob.set_val(Aircraft.Wing.AREA, 13.7)

        derivs = prob.check_partials(out_stream=None, method="cs")

        # TODO: need to test outputs too
        assert_check_partials(derivs, atol=1e-12, rtol=1e-12)


# region - dynamic test data taken from the baseline FLOPS output for each case
#    - first "# DETAILED FLIGHT SEGMENT SUMMARY"
#        - second "SEGMENT..."
#            - first three points
# NOTE: FLOPS bakes FCDO, FCDI, FCDSUB, and FCDSUP into the final reported drag
# coefficient - no intermediate calculations are reported in the "# DETAILED FLIGHT
# SEGMENT SUMMARY", so we have to back calculate what we can
def _add_drag_coefficients(
    case_name, dynamics_data: AviaryValues, M: np.ndarray, CD_scaled: np.ndarray,
    CD0_scaled: np.ndarray, CDI_scaled: np.ndarray
):
    '''
    Insert drag coefficients into the dynamics data, undoing FLOPS scaling.
    '''
    flops_inputs = get_flops_inputs(case_name)
    FCDSUB = flops_inputs.get_val(Aircraft.Design.SUBSONIC_DRAG_COEFF_FACTOR)
    FCDSUP = flops_inputs.get_val(Aircraft.Design.SUPERSONIC_DRAG_COEFF_FACTOR)

    idx_sup = np.where(M >= 1.0)
    CD = CD_scaled / FCDSUB
    CD[idx_sup] = CD_scaled[idx_sup] / FCDSUP
    dynamics_data.set_val('drag_coefficient', CD)

    FCD0 = flops_inputs.get_val(Aircraft.Design.ZERO_LIFT_DRAG_COEFF_FACTOR)
    CD0 = CD0_scaled / FCD0
    dynamics_data.set_val('CD0', CD0)

    FCDI = flops_inputs.get_val(Aircraft.Design.LIFT_DEPENDENT_DRAG_COEFF_FACTOR)
    CDI = CDI_scaled / FCDI
    dynamics_data.set_val('CDI', CDI)


dynamics_test_data = {}

key = 'LargeSingleAisle1FLOPS'
dynamics_test_data[key] = _dynamics_data = AviaryValues()
_dynamics_data.set_val(Dynamic.Mission.DYNAMIC_PRESSURE, np.array(
    [206.0, 205.6, 205.4]), 'lbf/ft**2')
_dynamics_data.set_val(Dynamic.Mission.MASS, np.array(
    [176751., 176400., 176185.]), 'lbm')
_dynamics_data.set_val(Dynamic.Mission.DRAG, np.array([9350., 9333., 9323.]), 'lbf')

M = np.array([0.7750, 0.7750, 0.7750])
CD_scaled = np.array([0.03313, 0.03313, 0.03313])
CD0_scaled = np.array([0.02012, 0.02013, 0.02013])
CDI_scaled = np.array([0.01301, 0.01301, 0.01300])

_dynamics_data.set_val(Dynamic.Mission.MACH, M)
_add_drag_coefficients(key, _dynamics_data, M, CD_scaled, CD0_scaled, CDI_scaled)

key = 'LargeSingleAisle2FLOPS'
dynamics_test_data[key] = _dynamics_data = AviaryValues()
_dynamics_data.set_val(Dynamic.Mission.DYNAMIC_PRESSURE, [
                       215.4, 215.4, 215.4], 'lbf/ft**2')
_dynamics_data.set_val(Dynamic.Mission.MASS, [169730., 169200., 167400.], 'lbm')
_dynamics_data.set_val(Dynamic.Mission.DRAG, [9542., 9512., 9411.], 'lbf')

M = np.array([0.7850, 0.7850, 0.7850])
CD_scaled = np.array([0.03304, 0.03293, 0.03258])
CD0_scaled = np.array([0.02016, 0.02016, 0.02016])
CDI_scaled = np.array([0.01288, 0.01277, 0.01242])

_dynamics_data.set_val(Dynamic.Mission.MACH, M)
_add_drag_coefficients(key, _dynamics_data, M, CD_scaled, CD0_scaled, CDI_scaled)

key = 'N3CC'
dynamics_test_data[key] = _dynamics_data = AviaryValues()
_dynamics_data.set_val(Dynamic.Mission.DYNAMIC_PRESSURE, [
                       208.4, 288.5, 364.0], 'lbf/ft**2')
_dynamics_data.set_val(Dynamic.Mission.MASS, [128777., 128721., 128667.], 'lbm')
_dynamics_data.set_val(Dynamic.Mission.DRAG, [5837., 6551., 7566.], 'lbf')

M = np.array([0.4522, 0.5321, 0.5985])
CD_scaled = np.array([0.02296, 0.01861, 0.01704])
CD0_scaled = np.array([0.01611, 0.01569, 0.01556])
CDI_scaled = np.array([0.00806, 0.00390, 0.00237])

_dynamics_data.set_val(Dynamic.Mission.MACH, M)
_add_drag_coefficients(key, _dynamics_data, M, CD_scaled, CD0_scaled, CDI_scaled)
# endregion - dynamic test data taken from the baseline FLOPS output for each case


if __name__ == "__main__":
    unittest.main()
